# :nodoc:
#
# Author:: Nathaniel Talbott.
# Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
# License:: Ruby license.

require 'test/unit/assertionfailederror'

module Test # :nodoc:
	module Unit # :nodoc:

		# Contains all of the standard Test::Unit assertions. Mixed in
		# to Test::Unit::TestCase. To mix it in and use its
		# functionality, you simply need to rescue
		# Test::Unit::AssertionFailedError, and you can additionally
		# override add_assertion to be notified whenever an assertion
		# is made.
		#
		# Notes:
		# * The message to each assertion, if given, will be
		#   propagated with the failure.
		# * It's easy to add your own assertions based on assert_block().
		module Assertions

			# The assertion upon which all other assertions are
			# based. Passes if the block yields true.
			def assert_block(message="") # :yields: 
				_wrap_assertion {
					if (! yield)
						raise AssertionFailedError.new(message.to_s)
					end
				}
			end

			# Passes if boolean is true.
			def assert(boolean, message="")
				_wrap_assertion {
					assert_block("assert should not be called with a block!") {!block_given?}
					assert_block(message) {boolean}
				}
			end

			# Passes if expected == actual. Note that the ordering of
			# arguments is important, since a helpful error message is
			# generated when this one fails that tells you the values
			# of expected and actual.
			def assert_equal(expected, actual, message=nil)
				_wrap_assertion {
					full_message = build_message(message, expected, actual) {
						| arg1, arg2 |
						"Expected <#{arg1}> but was <#{arg2}>"
					}
					assert_block(full_message) {
						expected == actual
					}
				}
			end

			# Passes if block raises exception.
			def assert_raises(expected_exception_klass, message="")
				_wrap_assertion {
					assert_instance_of(Class, expected_exception_klass, "Should expect a type of exception")
					actual_exception = nil
					full_message = build_message(message, expected_exception_klass) {
						| arg |
						"Expected exception of type <#{arg}> but none was thrown"
					}
					assert_block(full_message) {
						thrown = false
						begin
							yield
						rescue Exception => thrown_exception
							actual_exception = thrown_exception
							thrown = true
						end
						thrown
					}
					full_message = build_message(message, expected_exception_klass, actual_exception.class.name, actual_exception.message, actual_exception.backtrace.join("\n\t")) {
						| arg1, arg2, arg3, arg4 |
						"Expected exception to be of type <#{arg1}> but was <#{arg2}: #{arg3}\n\t#{arg4}>"
					}
					assert_block(full_message) {
						expected_exception_klass == actual_exception.class
					}
					actual_exception
				}
			end

			# Passes if object.class == klass.
			def assert_instance_of(klass, object, message="")
				_wrap_assertion {
					assert_equal(Class, klass.type, "assert_instance_of takes a Class as its first argument")
					full_message = build_message(message, object, klass, object.type) {
						| arg1, arg2, arg3 |
						"<#{arg1}> should have been an instance of <#{arg2}> but was <#{arg3}>"
					}
					assert_block(full_message) {
						klass == object.type
					}
				}
			end

			# Passes if object.nil?.
			def assert_nil(object, message="")
				assert_equal(nil, object, message)
			end

			# Passes if object.kind_of?(klass).
			def assert_kind_of(klass, object, message="")
				_wrap_assertion {
					assert(klass.kind_of?(Module), "The first parameter to assert_kind_of should be a kind_of Module.")
					full_message = build_message(message, object, klass) {
						| arg1, arg2 |
						"Expected <#{arg1}> to be kind_of?<#{arg2}>"
					}
					assert_block(full_message) {
						object.kind_of?(klass)
					}
				}
			end

			# Passes if object.respond_to?(method) is true.
			def assert_respond_to(object, method, message="")
				_wrap_assertion {
					full_message = build_message(message, object, method) {
						| arg1, arg2 |
						"Expected <#{arg1}> of type <#{Message.convert(object.class)}> to respond_to? <#{arg2}>"
					}
					assert_block(full_message) {
						object.respond_to?(method)
					}
				}
			end

			# Passes if string =~ regularExpression.
			def assert_match(regexp, string, message="")
				_wrap_assertion {
					assert_instance_of(Regexp, regexp, "The first argument to assert_match should be a Regexp.")
					full_message = build_message(message, string, regexp.source) {
						| arg1, arg2 |
						"Expected <#{arg1}> to match regexp </#{arg2}/>"
					}
					assert_block(full_message) {
						regexp =~ string
					}
				}
			end

			# Passes if actual.equal?(expected) (i.e. they are the
			# same instance).
			def assert_same(expected, actual, message="")
				_wrap_assertion {
					full_message = build_message(message, expected, expected.id, actual, actual.id) {
						| arg1, arg2, arg3, arg4 |
						"Expected <#{arg1}:#{arg2}> to be equal? to <#{arg3}:#{arg4}>"
					}
					assert_block(full_message) {
						actual.equal?(expected)
					}
				}
			end

			# Compares the two objects based on the passed
			# operator. Passes if object1.send(operator, object2) is
			# true.
			def assert_operator(object1, operator, object2, message="")
				_wrap_assertion {
					full_message = build_message(message, object1, operator, object2) {
						| arg1, arg2, arg3 |
						"Expected <#{arg1}> to be #{arg2} <#{arg3}>"
					}
					assert_block(full_message) {
						object1.send(operator, object2)
					}
				}
			end

			# Passes if block does not raise an exception.
			def assert_nothing_raised(*args)
				_wrap_assertion {
					message = ""
					if (!args[-1].instance_of?(Class))
						message = args.pop
					end
					begin
						yield
					rescue Exception => thrown_exception
						if (args.empty? || args.include?(thrown_exception.class))
							full_message = build_message(message, thrown_exception.class.name, thrown_exception.message, thrown_exception.backtrace.join("\n\t")) {
								| arg1, arg2, arg3 |
								"Exception raised: <#{arg1}: #{arg2}\n\t#{arg3}>"
							}
							flunk(full_message)
						else
							raise thrown_exception.class, thrown_exception.message, thrown_exception.backtrace
						end
					end
				}
			end

			# Always fails.
			def flunk(message="")
				assert(false, message)
			end

			# Passes if !actual.equal?(expected).
			def assert_not_same(expected, actual, message="")
				_wrap_assertion {
					full_message = build_message(message, expected, expected.id, actual, actual.id) {
						| arg1, arg2, arg3, arg4 |
						"Expected <#{arg1}:#{arg2}> to not be equal? to <#{arg3}:#{arg4}>"
					}
					assert_block(full_message) {
						! actual.equal?(expected)
					}
				}
			end

			# Passes if expected != actual.
			def assert_not_equal(expected, actual, message="")
				_wrap_assertion {
					full_message = build_message(message, expected, actual) {
						| arg1, arg2 |
						"Expected <#{arg1}> to be != to <#{arg2}>"
					}
					assert_block(full_message) {
						expected != actual
					}
				}
			end

			# Passes if !object.nil?.
			def assert_not_nil(object, message="")
				_wrap_assertion {
					full_message = build_message(message, object) {
						| arg |
						"Expected <#{arg}> to not be nil"
					}
					assert_block(full_message) {
						! object.nil?
					}
				}
			end

			# Passes if string !~ regularExpression.
			def assert_does_not_match(regexp, string, message="")
				_wrap_assertion {
					assert_instance_of(Regexp, regexp, "The first argument to assertDoesNotMatch should be a Regexp.")
					full_message = build_message(message, regexp.source, string) {
						| arg1, arg2 |
						"Expected </#{arg1}/> to not match <#{arg2}>"
					}
					assert_block(full_message) {
						regexp !~ string
					}
				}
			end

			# Passes if block throws symbol.
			def assert_throws(expected_symbol, message="", &proc)
				_wrap_assertion {
					assert_instance_of(Symbol, expected_symbol, "assertThrows expects the symbol that should be thrown for its first argument")
					assert(block_given?, "Should have passed a block to assertThrows")
					caught = true
					begin
						catch(expected_symbol) {
							proc.call
							caught = false
						}
						full_message = build_message(message, expected_symbol) {
							| arg |
							"<:#{arg}> should have been thrown"
						}
						assert(caught, full_message)
					rescue NameError => name_error
						if ( name_error.message !~ /^uncaught throw `(.+)'$/ )
							raise name_error
						end
						full_message = build_message(message, expected_symbol, $1) {
							| arg1, arg2 |
							"Expected <:#{arg1}> to be thrown but <:#{arg2}> was thrown"
						}
						flunk(full_message)
					end	
				}
			end

			# Passes if block does not throw anything.
			def assert_nothing_thrown(message="", &proc)
				_wrap_assertion {
					assert(block_given?, "Should have passed a block to assertNothingThrown")
					begin
						proc.call
					rescue NameError => name_error
						if (name_error.message !~ /^uncaught throw `(.+)'$/ )
							raise name_error
						end
						full_message = build_message(message, $1) {
							| arg |
							"Nothing should have been thrown but <:#{arg}> was thrown"
						}
						flunk(full_message)
					end
					full_message = build_message(message) { ||
						"Nothing should have been thrown"
					}
					assert(true, full_message)
				}
			end

			# Passes if expected_float and actual_float are equal
			# within delta tolerance.
			def assert_in_delta(expected_float, actual_float, delta, message="")
				_wrap_assertion {
					{expected_float => "first float", actual_float => "second float", delta => "delta"}.each {
						|float, name|
						assert_respond_to(float, :to_f, "The arguments must respond to to_f; the #{name} did not")
					}
					full_message = build_message(message, expected_float, actual_float, delta) {
						|arg1, arg2, arg3|
						"Expected <#{arg1}> and <#{arg2}> to be within <#{arg3}> of each other"
					}
					assert_block(full_message) {
						(expected_float - actual_float).abs < delta.abs
					}
				}
			end

			# Passes if the method sent returns a true value.
			def assert_send(send_array, message="")
				_wrap_assertion {
					assert_instance_of(Array, send_array, "assert_send requires an array of send information")
					assert(send_array.size >= 2, "assert_send requires at least a receiver and a message name")
					full_message = build_message(message, send_array[0], send_array[1], send_array[2..-1]) {
						|arg1, arg2, arg3|
						"Expected <#{arg1}> to respond to <#{arg2}(#{arg3})> with true"
					}
					assert_block(full_message) {
						send_array[0].__send__(send_array[1], *send_array[2..-1])
					}
				}
			end

			def build_message(message, *arguments, &block) # :nodoc:
				assert_block("Should have passed a block to build_message") {
					block_given?
				}
				proc_arity = ((block.arity == -1) ? 1 : block.arity)
				assert_block("Should have passed the same number of arguments as the message block expects. Expected <#{proc_arity}> but was <#{arguments.length}>") {
					proc_arity == arguments.length
				}
				return Message.new(message.to_s, arguments, &block)
			end

			def _wrap_assertion # :nodoc:
				@_assertion_wrapped ||= false
				if (!@_assertion_wrapped)
					@_assertion_wrapped = true
					begin
						add_assertion
						return yield
					ensure
						@_assertion_wrapped = false
					end
				else
					return yield
				end
			end
			private :_wrap_assertion
			
			# Called whenever an assertion is made.
			def add_assertion
			end
			private :add_assertion
			
			class Message # :nodoc: all
				def self.convert(object)
					case object
						when String
							return object
						when Symbol
							return object.to_s
						else
							return object.inspect
					end
				end
				def initialize(message, parameters, &block)
					@message = message
					@parameters = parameters
					@block = block
				end
				def to_s
					message_parts = []
					if (@message != nil && @message != "")
						if (@message !~ /\.$/)
							@message << "."
						end
						message_parts << @message
					end
					@parameters = @parameters.collect {
						| parameter |
						self.class.convert(parameter)
					}
					message_parts << @block.call(*@parameters)
					return message_parts.join(" ")
				end
			end
		end
	end
end
