# :nodoc:
#
# Author:: Nathaniel Talbott.
# Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
# License:: Ruby license.

require 'fox'
require 'fox/responder'
require 'test/unit/ui/testrunnermediator'
require 'test/unit/ui/testrunnerutilities'

include Fox

GC.disable

module Test
	module Unit
		module UI
			module Fox # :nodoc:

				# Runs a Test::Unit::TestSuite in a Fox UI. Obviously,
				# this one requires you to have Fox
				# (http://www.fox-toolkit.org/fox.html) and the Ruby
				# Fox extension (http://fxruby.sourceforge.net/)
				# installed.
				class TestRunner < FXMainWindow
					include Responder
					extend TestRunnerUtilities
	
					ID_RUN, ID_SELECT_LISTITEM, ID_QUIT = 
						(FXMainWindow::ID_LAST...(FXMainWindow::ID_LAST+3)).to_a

					# Creates a new TestRunner and runs the suite.
					def TestRunner.run(suite)
						new(suite).start
					end

					# Creates a new TestRunner for running the passed
					# suite.
					def initialize(suite)
						super(main_app, "R2 TestRunner", nil, nil, DECOR_ALL, 20, 20)
	
						FXTooltip.new(main_app)
	
						@suite = suite
	
						FXMAPFUNC(SEL_COMMAND, ID_RUN, "on_run")
						FXMAPFUNC(SEL_COMMAND, ID_SELECT_LISTITEM, "on_select_list_item")
						FXMAPFUNC(SEL_COMMAND, ID_QUIT, "on_quit")
	
						@red = false
	
						main_panel
					end
	
					# Begins the test run.
					def start
						setup_ui
						setup_mediator
						attach_to_mediator
						start_ui
					end

					private
					def on_run(sender, sel, ptr) # :nodoc:
						@mediator.run_suite
						1
					end
	
					def on_quit(sender, sel, ptr) # :nodoc:
					end
	
					def on_select_list_item(sender, sel, ptr) # :nodoc:
						if (sender.retrieveItem(sender.getCurrentItem).isSelected)
							show_fault(sender.retrieveItem(sender.getCurrentItem).fault)
						else
							clear_fault
						end
						1
					end
	
					def setup_mediator # :nodoc:
						@mediator = TestRunnerMediator.new(@suite)
						suite_name = @suite.to_s
						if ( @suite.kind_of?(Module) )
							suite_name = @suite.name
						end
						suite_name_entry.setText(suite_name)
					end
					def setup_ui # :nodoc:
						@red = false
					end
					def attach_to_mediator # :nodoc:
						@mediator.add_listener(TestRunnerMediator::RESET, &method(:reset_ui))
						@mediator.add_listener(TestRunnerMediator::FAULT_ADDED, &method(:add_fault))
						@mediator.add_listener(TestRunnerMediator::RESULT_CHANGED, &method(:result_changed))
						@mediator.add_listener(TestRunnerMediator::STATUS_CHANGED, &method(:update_status))
					end
	
					def create # :nodoc:
						super
						show(PLACEMENT_SCREEN)
					end
	
					def start_ui # :nodoc:
						main_app.create
						main_app.run
					end
					def stop # :nodoc:
						main_app.exit(0)
					end
					def reset_ui(count) # :nodoc:
						test_progress_bar.setBarColor(green_style)
						test_progress_bar.setTotal(count)
						test_progress_bar.setProgress(0)
						@red = false
	
						runCountLabel.setText("0")
						assertionCountLabel.setText("0")
						failureCountLabel.setText("0")
						errorCountLabel.setText("0")
	
						faultList.clearItems
					end
					def add_fault(fault) # :nodoc:
						if ( ! @red )
							test_progress_bar.setBarColor(red_style)
							@red = true
						end
						item = FaultListItem.new(fault)
						faultList.appendItem(item)
					end
					def show_fault(fault) # :nodoc:
						raw_show_fault(fault.longDisplay)
					end
					def raw_show_fault(string) # :nodoc:
						faultDetailLabel.setText(string)
	#					outerDetailSubPanel.queue_resize
					end
					def clear_fault # :nodoc:
						raw_show_fault("")
					end
					def result_changed(result) # :nodoc:
						test_progress_bar.setProgress(test_progress_bar.getProgress + 1)
	
						run_count_label.setText(result.runs.to_s)
						assertion_count_label.setText(result.assertions.to_s)
						failure_count_label.setText(result.failures.to_s)
						error_count_label.setText(result.errors.to_s)
					end
					def update_status(status) # :nodoc:
						status_entry.getStatusline.setText(status.message)
					end
	
					def main_app # :nodoc:
						lazy_initialize(:main_app) {
							@main_app = FXApp.new("TestRunner", "Test::Unit")
							@main_app.init([])
						}
					end
	
					def main_panel # :nodoc:
						lazy_initialize(:main_panel) {
							@main_panel = FXVerticalFrame.new(self, LAYOUT_FILL_X)
							suite_panel
							progress_panel
							info_panel
							list_panel
							detail_panel
							status_entry
						}
					end
	
					def suite_panel # :nodoc:
						lazy_initialize(:suite_panel) {
							@suite_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X, 0,0,0,0,10,10,10)
							FXLabel.new(@suite_panel, "Suite:", nil, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X)
							suite_name_entry
							run_button
						}
					end
					def suite_name_entry # :nodoc:
						lazy_initialize(:suite_name_entry) {
							@suite_name_entry = FXTextField.new(@suite_panel, 30)
							@suite_name_entry.disable
						}
					end
					def run_button # :nodoc:
						lazy_initialize(:run_button) {
							@run_button = FXButton.new(@suite_panel, "&Run\tRun the current suite", nil, self, ID_RUN)
						}
					end
					def progress_panel # :nodoc:
						lazy_initialize(:progress_panel) {
							@progress_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 50, 10,10,10)
							test_progress_bar
						}
					end
					def test_progress_bar # :nodoc:
						lazy_initialize(:test_progress_bar) {
							@test_progress_bar = FXProgressBar.new(@progress_panel, nil, 0, PROGRESSBAR_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_Y)#, 0, 0, 0, 50)
							@test_progress_bar.setHeight(500)
							@test_progress_bar.setBarColor(green_style)
						}
					end
					def green_style # :nodoc:
						lazy_initialize(:green_style) {
							@green_style = FXRGBA(0,0xFF,0,0xFF) #0x00FF0000
						}
					end
					def red_style # :nodoc:
						lazy_initialize(:red_style) {
							@red_style = FXRGBA(0xFF,0,0,0xFF) #0xFF000000
						}
					end
					def info_panel # :nodoc:
						lazy_initialize(:info_panel) {
							@info_panel = FXHorizontalFrame.new(@main_panel, 0, 0,0,0,0, 10,10,10)
							FXLabel.new(@info_panel, "Runs:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
							run_count_label
							FXLabel.new(@info_panel, "Assertions:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
							assertion_count_label
							FXLabel.new(@info_panel, "Failures:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
							failure_count_label
							FXLabel.new(@info_panel, "Errors:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
							error_count_label
						}
					end
					def run_count_label # :nodoc:
						lazy_initialize(:run_count_label) {
							@run_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
						}
					end
					def assertion_count_label # :nodoc:
						lazy_initialize(:assertion_count_label) {
							@assertion_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
						}
					end
					def failure_count_label # :nodoc:
						lazy_initialize(:failure_count_label) {
							@failure_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
						}
					end
					def error_count_label # :nodoc:
						lazy_initialize(:error_count_label) {
							@error_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_X)
						}
					end
					def list_panel # :nodoc:
						lazy_initialize(:list_panel) {
							@list_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_FILL_X | FRAME_SUNKEN | FRAME_THICK, 0,0,0,0, 10,10,10)
							fault_list
						}
					end
					def fault_list # :nodoc:
						lazy_initialize(:fault_list) {
							@fault_list = FXList.new(@list_panel, 10, self, ID_SELECT_LISTITEM, LIST_SINGLESELECT | LAYOUT_FILL_X) #, 0, 0, 0, 150)
						}
					end
					def detail_panel # :nodoc:
						lazy_initialize(:detail_panel) {
							@detail_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_FILL_X | FRAME_SUNKEN | FRAME_THICK, 0,0,0,0, 10,10,10)
							detail_scrolled_window
						}
					end
					def detail_scrolled_window # :nodoc:
						lazy_initialize(:detail_scrolled_window) {
							@detail_scrolled_window = FXScrollWindow.new(@detail_panel, LAYOUT_FILL_X)
							fault_detail_label
						}
					end
					def fault_detail_label # :nodoc:
						lazy_initialize(:fault_detail_label) {
							@fault_detail_label = FXLabel.new(@detail_scrolled_window, "", nil, LAYOUT_SIDE_LEFT)
							class << @fault_detail_label # :nodoc:
								def setText(text)
									super(text.gsub(/\n\t/, "\n" + (" " * 4)))
								end
							end
						}
					end
					def statusPanel # :nodoc:
						lazy_initialize(:statusPanel) {
							@statusPanel = FXHorizontalFrame.new(@mainWindow)
							status_entry
						}
					end
					def status_entry # :nodoc:
						lazy_initialize(:status_entry) {
							@status_entry = FXStatusbar.new(self, LAYOUT_SIDE_RIGHT | LAYOUT_FILL_X)
						}
					end
	
					def lazy_initialize(symbol) # :nodoc:
						if ( instance_eval("@" + symbol.to_s).nil? )
							yield
						end
						return instance_eval("@" + symbol.to_s)
					end
				end
	
				class FaultListItem < FXListItem # :nodoc: all
					attr_reader(:fault)
					def initialize(fault)
						super(fault.short_display)
						@fault = fault
					end
				end
			end
		end
	end
end

if __FILE__ == $0
	Test::Unit::UI::Fox::TestRunner.start_command_line_test
end
