# :nodoc:
#
# Author:: Nathaniel Talbott.
# Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
# License:: Ruby license.

require 'test/unit/ui/util/observable'
require 'test/unit/testresult'

module Test
	module Unit
		module UI # :nodoc:

			# Provides an interface to write any given UI against,
			# hopefully making it easy to write new UIs.
			class TestRunnerMediator
				RESET = "TestRunnerMediator::RESET"
				STATUS_CHANGED = "TestRunnerMediator::STATUS_CHANGED"
				RESULT_CHANGED = "TestRunnerMediator::RESULT_CHANGED"
				FAULT_ADDED = "TestRunnerMediator::FAULT_ADDED"
				
				include Util::Observable
				
				attr_reader(:tuner)
	
				@@run = false

				# Returns true if any TestRunnerMediator instances
				# have been run.
				def TestRunnerMediator.run?
					return @@run
				end

				# Creates a new TestRunnerMediator initialized to run
				# the passed suite.
				def initialize(suite)
					@suite = suite
				end

				# Runs the suite the TestRunnerMediator was created
				# with.
				def run_suite
					@@run = true
					begin_time = Time.now
					notify_listeners(RESET, @suite.size)
					notify_listeners(STATUS_CHANGED, Status.new(Status::STARTED_RUNNING, "Started..."))
					result = create_result
					result_listener = result.add_listener(TestResult::CHANGED) {
						| updated_result |
						notify_listeners(RESULT_CHANGED, updated_result)
					}
					fault_listener = result.add_listener(TestResult::FAULT) {
						| fault |
						notify_listeners(FAULT_ADDED, fault)
					}
					@suite.run(result) {
						| progress |
						notify_listeners(STATUS_CHANGED, Status.new(Status::FINISHED_TEST, progress))
					}
					result.remove_listener(TestResult::FAULT, fault_listener)
					result.remove_listener(TestResult::CHANGED, result_listener)
					end_time = Time.now
					elapsed_time = end_time - begin_time
					notify_listeners(STATUS_CHANGED, Status.new(Status::FINISHED_RUNNING, "Finished in #{elapsed_time} seconds."))
					return result
				end

				private
				# A factory method to create the result the mediator
				# should run with. Can be overridden by subclasses if
				# one wants to use a different result.
				def create_result
					return TestResult.new
				end
			end

			# Used by TestRunnerMediator to pass status information to
			# its runner.
			class Status
				STARTED_RUNNING = "Status::STARTED_RUNNING"
				FINISHED_TEST = "Status::FINISHED_TEST"
				FINISHED_RUNNING = "Status::FINISHED_RUNNING"
				attr_reader(:type, :message)

				# Create a new status message with the given type and
				# message.
				def initialize(type, message)
					@type = type
					@message = message
				end
			end
		end
	end
end
